# -*- coding:Utf-8 -*-

# - A kód a holecircle.py állományból átemelt részleteket tartalmaz

# Menetvágás rugalmas paraméterezéssel, 1 vagy 2 bekezdés, simító fogás megadás lehetőséggel

import sys, os
try:
    BASE = os.environ['EMC2_HOME']
except:
    BASE = os.environ['LINUXCNC_HOME']

sys.path.insert(0, os.path.join(BASE, "lib", "python"))

from math import sqrt, pi, tan

def _(s): return s

def ui():
    import Tkinter
    import pickle
    import nf
    import rs274.options
    import os

    app = Tkinter.Tk()
    rs274.options.install(app)
    app.tk.call("source", os.path.join(BASE, "share", "axis", "tcl", "combobox.tcl"))

    app.wm_title(_("Menetvágás V0.21"))
    app.wm_iconname(_("Menetvágás V0.21"))

    prev = Tkinter.Canvas(app, width=0, height=300)
    f = Tkinter.Frame(app)
    b = Tkinter.Frame(app)
    prev.grid(row=0, column=0, sticky="nw")
    f.grid(row=0, column=1, sticky="nw")
    b.grid(row=1, column=0, columnspan=2, sticky="ne")

    validate_float    = "expr {[regexp {^-?([0-9]+(\.[0-9]*)?|\.[0-9]+|)$} %P]}"
    validate_int      = "expr {[regexp {^-?([0-9]+|)$} %P]}"
    validate_posfloat = "expr {[regexp {^?([0-9]+(\.[0-9]*)?|\.[0-9]+|)$} %P]}"
    validate_posint   = "expr {[regexp {^([0-9]+|)$} %P]}"
    def posfloatentry(f, v):
        var = Tkinter.DoubleVar(f)
        var.set(v)
        w = Tkinter.Entry(f, textvariable=var, validatecommand=validate_posfloat, validate="all", width=10)
        return w, var

    def floatentry(f, v):
        var = Tkinter.DoubleVar(f)
        var.set(v)
        w = Tkinter.Entry(f, textvariable=var, validatecommand=validate_float, validate="all", width=10)
        return w, var

    def posintentry(f, v):
        var = Tkinter.IntVar(f)
        var.set(v)
        w = Tkinter.Entry(f, textvariable=var, validatecommand=validate_posint, validate="all", width=10)
        return w, var


    def intentry(f, v):
        var = Tkinter.IntVar(f)
        var.set(v)
        w = Tkinter.Entry(f, textvariable=var, validatecommand=validate_int, validate="all", width=10)
        return w, var

    def checkbutton(k, v):
        var = Tkinter.BooleanVar(f)
        var.set(v)
        g = Tkinter.Frame(f)
        w = Tkinter.Checkbutton(g, variable=var, text="Yes")
        w.pack(side="left")
        return g, var 

    def intscale(k, v, min=1, max = 100):
        var = Tkinter.IntVar(f)
        var.set(v)
        g = Tkinter.Frame(f, borderwidth=0)
        w = Tkinter.Scale(g, orient="h", variable=var, from_=min, to=max, showvalue=False)
        l = Tkinter.Label(g, textvariable=var, width=3)
        l.pack(side="left")
        w.pack(side="left", fill="x", expand=1)
        return g, var

    def optionmenu(k, v, *options):
        options = list(options)
        def trace(*args):
            try:
                var.set(options.index(svar.get()))
            except ValueError:
                pass

        try:
            opt = options[v]
        except (TypeError, IndexError):
            v = 0
            opt = options[0]

        var = Tkinter.IntVar(f)    
        var.set(v)
        svar = Tkinter.StringVar(f)
        svar.set(options[v])
        svar.trace("w", trace)
        wp = f._w.rstrip(".") + ".c" + svar._name
        f.tk.call("combobox::combobox", wp, "-editable", 0, "-width",
                max(len(opt) for opt in options)+3, "-textvariable", svar._name,
                "-background", "white")
        f.tk.call(wp, "list", "insert", "end", *options)
        w = nf.makewidget(f, Tkinter.Widget, wp)
        return w, var

    rc = os.path.expanduser("~/.tobbbekmenet")
    constructors = [
        ("tool", posintentry),        
        ("forgi", lambda f, v: optionmenu(f, v, _("M3"), _("M4"))),
        ("hut", lambda f, v: optionmenu(f, v, _("M7"), _("M8"),_("M9"))),
        ("ford", posintentry),
        ("wait", posfloatentry), 
        ("xk", floatentry),
        ("xv", floatentry),
        ("zk", floatentry),
        ("zv", floatentry),
	      ("mm", posfloatentry),
	      ("pz", posfloatentry),
	      ("kf", posfloatentry),
	      ("lkf", posfloatentry),
	      ("fcs", posfloatentry),
	      ("sz", floatentry),
        ("beksz", lambda f, v: optionmenu(f, v, _("1"), _("2"))),
        ("zmod", floatentry),
        ("sfsz", lambda f, v: optionmenu(f, v, _("0"), _("1"), _("2"))),
        ("elhat", lambda f, v: optionmenu(f, v, _("X+"), _("X-"))),
    ]

    defaults = dict(
      tool= 1,
      elhat= 0,
      forgi= 0,
      wait= 2,
      ford= 400,
	    xk = 0,
	    xv = 20,
	    zk = 8,
	    zv = -40,
	    mm = 2,
	    pz = 6,
	    kf= 0.4,
	    lkf= 0.05,
	    fcs= 95,
	    sz= 29.5,
	    beksz= 1,
      zmod= 0,
      sfsz= 0,
      hut= 1,
    )

    texts = dict(
        tool=_("Szerszám száma"),
        elhat=_("Kés elhelyezése"),
        forgi=_("Forgásirány"),
        hut=_("Hűtés"),
        ford=_("Fordulatszám"),
        wait=_("Felpörgés ideje"),
	      xk=_("X kezdő átmérő"),
	      xv=_("X befejező átmérő"),
	      zk=_("Z kezdő méret"),
	      zv=_("Z befejező méret"),
        mm=_("Menetmélység"),
	      pz=_("Emelkedés Z tengelyen"),
	      kf=_("Kezdő fogás"),
	      lkf=_("Legkisebb fogás"),
	      fcs=_("Fogás csökkentés %-ban"),
	      sz=_("Fogásvétel szöge"),
	      beksz=_("Bekezdések száma"),
        sfsz=_("Simító fogások száma"),
        zmod=_("2. bekezdés Z korrekció"),
    )

    try:
        defaults.update(pickle.load(open(rc, "rb")))
    except (IOError, pickle.PickleError): pass

    vars = {}
    widgets = {}
    for j, (k, con) in enumerate(constructors):
        v = defaults[k]
        text = texts.get(k, k.replace("_", " "))
        lab = Tkinter.Label(f, text=text)
        widgets[k], vars[k] = con(f, v)
        lab.grid(row=j, column=0, sticky="w")
        widgets[k].grid(row=j, column=1, sticky="ew")

    status = Tkinter.IntVar()
    bb = Tkinter.Button(b, text=_("Kód generálás"), command=lambda:status.set(1), width=12, default="active")
    bb.pack(side="left", padx=4, pady=4)

    
    app.bind("<Escape>", lambda evt: bb.invoke())
    app.bind("<Return>", lambda evt: bb.invoke())
    app.wm_protocol("WM_DELETE_WINDOW", lambda: bb.invoke())
    app.wm_resizable(0,0)

    app.wait_visibility()
    app.tk.call("after", "idle", ("after", "idle", "focus [tk_focusNext .]"))
    app.wait_variable(status)

    if status.get() == -1:
	    raise SystemExit(1)

    for k, v in vars.items():
        defaults[k] = v.get()

    app.destroy()

    pickle.dump(defaults, open(rc, "wb"))

    return defaults

u = ui()

tool = u['tool']
elhat = u['elhat']
xk = u['xk']
xv = u['xv']
zk = u['zk']
zv = u['zv']
mm = abs(u['mm'])
pz = abs(u['pz'])
kf = abs(u['kf'])
lkf = abs(u['lkf'])
fcs = abs(u['fcs'])
sz = u['sz']
ford = abs(u['ford'])
wait = abs(u['wait'])
beksz = u['beksz']
zmod = u['zmod']
sfsz = u['sfsz']
hut = u['hut']

if elhat == 0:
    elj= 1
else:
    elj= -1

sf_= sfsz

xkr=    (xk/2)            #(ATSZAMITAS RADIUSZRA)
xvr=    (xv/2)            #(ATSZAMITAS RADIUSZRA)
dx=     (xvr-xkr)         #(DELTA X)
dz=     abs(zv-zk)        #(DELTA Z)
pp=     sqrt(dx*dx+dz*dz)
pp=     (pp/dz*pz)        #(MENETEMELKEDES A PALASTON)
pz=     (pz+zmod)         #(MODOSITOTT EMELKEDES - CSAK KORREKCIOHOZ HASZNALVA!)
zkorr=  (mm*tan(sz*pi/180))      #(Z VEGPONT KORREKCIO)
xkorr=  (zkorr*dx/dz)     #(X VEGPONT KORREKCIO)
tbekx=  (pz/2*dx/dz)      #(II. BEKEZDES X ELTOLAS)


print "(Menetvágás V0.21)"
print "G18 G90 G95 G40 G7"
print "T",(tool),"M6 G43"

spincodes = ['M3', 'M4']
coolcodes = ['M7', 'M8', 'M9']
print "S", ford, spincodes[u['forgi']]

if hut <> 2:
  print coolcodes[u['hut']]

if wait > 0:
  print "G4 P% .1f" % (wait)

print "G0 Z% +.3f" % (zk)
print "X% +.3f" % (xk*elj)
		
aktxk=  xkr               #(AKTUALIS X KEZDO)
aktxv=  xvr               #(AKTUALIS X VEG)
flag=   0

while ((flag == 0) or (sfsz <>0)):
  print "G0 X% +.3f" % (xk*elj)

  if (kf < lkf):
      kf= lkf

  aktxk-= kf
  aktxv-= kf
  kf=kf*fcs/100

  if aktxk <= (xkr-mm):
      aktxk= xkr-mm     #(HA ELERTE A MENETMELYSEGET)
      aktxv= xvr-mm
      if ((sfsz > 0) and (flag == 5)):
          sfsz-= 1
          print "(Simító fogás:" ,sf_-sfsz, ".)"
      flag=5

  zkorrsz= (xkr-aktxk)*tan(sz*pi/180)
        #(I. BEKEZDES)
  print "G0 X% +.3f Z% +.3f" % ( 2*aktxk*elj, zk-zkorrsz)
  print "G33 X% +.3f Z% +.3f K% +.3f" % ( 2*(aktxv-xkorr)*elj, zv-zkorrsz+zkorr, pp)
  print "G33 X% +.3f K% +.3f" % ((xv+4)*elj, pp/2)

  if beksz ==0:
      print "G0 Z% +.3f" % (zk)
  else:
      print "G0 Z% +.3f" % (zk+pz/2)
      print "X% +.3f" % ((xk-2*tbekx)*elj)
        #(II. BEKEZDES)
      print "X% +.3f Z% +.3f" % ( 2*(aktxk-tbekx)*elj, zk-zkorrsz+pz/2)
      print "G33 X% +.3f Z% +.3f K% +.3f" % ( 2*(aktxv-xkorr)*elj, zv-zkorrsz+zkorr, pp)
      print "G33 X% +.3f K% +.3f" % ((xv+4)*elj, pp/2)
      print "G0 Z% +.3f" % (zk)

if hut <> 2:
  print coolcodes[2]

print "M5"
print "G53 X-10 Z-10"   #(Kiállás - ezt beállítani!)
print "M30"

